@@ -23,12 +23,8 @@
 #include <lua.h>
 #include <lauxlib.h>
 
-#if LUA_VERSION_NUM < 502
-/* lua_rawlen: Not entirely correct, but should work anyway */
-#  define lua_rawlen lua_objlen
-#  define luaL_newlib(L,l) (lua_newtable(L), luaL_register(L,NULL,l))
-#  define luaL_setfuncs(L,l,n) (assert(n==0), luaL_register(L,NULL,l))
-#endif
+#include "compat-5.3.h"
+
 
 #ifdef WINLDAPAPI
 #define timeval l_timeval
@@ -726,11 +722,8 @@ static int next_message (lua_State *L) {
 	int rc;
 	int ret;
 
-	luaL_checktype(L, 1, LUA_TTABLE);
-
 	lua_rawgeti (L, LUA_REGISTRYINDEX, search->conn);
 	conn = (conn_data *)lua_touserdata (L, -1); /* get connection */
-
 	rc = ldap_result (conn->ld, search->msgid, LDAP_MSG_ONE, get_timeout_param (L, 1, &timeout), &res);
 	if (rc == 0)
 		return faildirect (L, LUALDAP_PREFIX"result timeout expired");
@@ -889,7 +882,7 @@ static int lualdap_conn_tostring (lua_State *L) {
 	if (conn->ld == NULL)
 		strcpy (buff, "closed");
 	else
-		sprintf (buff, "%p", conn);
+		sprintf (buff, "%p", (void*)conn);
 	lua_pushfstring (L, "%s (%s)", LUALDAP_CONNECTION_METATABLE, buff);
 	return 1;
 }
@@ -906,7 +899,7 @@ static int lualdap_search_tostring (lua_State *L) {
 	if (search->conn == LUA_NOREF)
 		strcpy (buff, "closed");
 	else
-		sprintf (buff, "%p", search);
+		sprintf (buff, "%p", (void*)search);
 	lua_pushfstring (L, "%s (%s)", LUALDAP_SEARCH_METATABLE, buff);
 	return 1;
 }
@@ -1009,12 +1002,11 @@ static int lualdap_initialize (lua_State *L) {
 static int lualdap_open_simple (lua_State *L) {
 	ldap_pchar_t host = (ldap_pchar_t) luaL_checkstring (L, 1);
 	ldap_pchar_t who = (ldap_pchar_t) luaL_optstring (L, 2, NULL);
-	const char *password = luaL_optstring (L, 3, NULL);
+	const char *password = luaL_optstring (L, 3, "");
 	int use_tls = lua_toboolean (L, 4);
 	conn_data *conn = (conn_data *)lua_newuserdata (L, sizeof(conn_data));
 #if defined(LDAP_API_FEATURE_X_OPENLDAP) && LDAP_API_FEATURE_X_OPENLDAP >= 20300
-	struct berval cred = { 0, NULL };
-	char *host_with_schema = NULL;
+	struct berval *cred = NULL;
 #endif
 	int err;
 
@@ -1022,12 +1014,16 @@ static int lualdap_open_simple (lua_State *L) {
 	lualdap_setmeta (L, LUALDAP_CONNECTION_METATABLE);
 	conn->version = 0;
 #if defined(LDAP_API_FEATURE_X_OPENLDAP) && LDAP_API_FEATURE_X_OPENLDAP >= 20300
-	host_with_schema = malloc(strlen(host) + 8);
-	strcpy(host_with_schema, "ldap://");
-	strcat(host_with_schema, host);
-	err = ldap_initialize(&conn->ld, host_with_schema);
-	free(host_with_schema);
-	host_with_schema = NULL;
+	if (strstr(host, "://") != NULL) {
+		err = ldap_initialize(&conn->ld, host);
+	} else {
+		char *host_with_schema = malloc(strlen(host) + 8);
+		strcpy(host_with_schema, "ldap://");
+		strcat(host_with_schema, host);
+		err = ldap_initialize(&conn->ld, host_with_schema);
+		free(host_with_schema);
+		host_with_schema = NULL;
+	}
 	if (err != LDAP_SUCCESS)
 #else
 	conn->ld = ldap_init (host, LDAP_PORT);
@@ -1047,12 +1043,9 @@ static int lualdap_open_simple (lua_State *L) {
 	}
 	/* Bind to a server */
 #if defined(LDAP_API_FEATURE_X_OPENLDAP) && LDAP_API_FEATURE_X_OPENLDAP >= 20300
-	cred.bv_len = strlen(password);
-	cred.bv_val = malloc(cred.bv_len+1);
-	strcpy(cred.bv_val, password);
-	err = ldap_sasl_bind_s (conn->ld, who, LDAP_SASL_SIMPLE, &cred, NULL, NULL, NULL);
-	free(cred.bv_val);
-	memset(&cred, 0, sizeof(cred));
+	cred = ber_bvstrdup(password);
+	err = ldap_sasl_bind_s (conn->ld, who, LDAP_SASL_SIMPLE, cred, NULL, NULL, NULL);
+	ber_bvfree(cred);
 #else
 	err = ldap_bind_s (conn->ld, who, password, LDAP_AUTH_SIMPLE);
 #endif
@@ -1074,7 +1067,7 @@ static void set_info (lua_State *L) {
 	lua_pushliteral (L, "LuaLDAP is a simple interface from Lua to an LDAP client");
 	lua_settable (L, -3);
 	lua_pushliteral (L, "_VERSION");
-	lua_pushliteral (L, "LuaLDAP 1.2.2");
+	lua_pushliteral (L, PACKAGE_STRING);
 	lua_settable (L, -3);
 }
 
